/* ---------------------------------------------------------- */
/*  kome_audio_drv.c                                          */
/* ---------------------------------------------------------- */

/*------------------------------------------------------------- */
/** @file
    @brief		V850/DAC audio driver
    @author		rerofumi
    @since		Jul.13.2007
*/
/*-----------------------------------------------------
 Copyright (C) 2007 rerofumi <rero2@yuumu.org>
 -  Kome wo kame.
 ------------------------------------------------------*/

/*-------------------------------*/
/* include                       */
/*-------------------------------*/

#include "kome_audio_drv.h"

/*-------------------------------*/
/* define                        */
/*-------------------------------*/

/*-------------------------------*/
/* struct                        */
/*-------------------------------*/

/*-------------------------------*/
/* global value                  */
/*-------------------------------*/

//! ̃obt@TCh (0 or 1)
int  audio_buffer_side;

//! o̓obt@
unsigned char  audio_buffer[KOMEAUDIO_BUFFERSIZE*2];

//! o̓`l
KAudioChannel  se_channel[KOMEAUDIO_CHANNEL];

//! AZuWJp
unsigned long *work_buffer;
unsigned long *work_data;
int  work_data_rest, work_data_size;
int  work_loop;

/*-------------------------------*/
/* local routine                 */
/*-------------------------------*/

//! mixing Ȃ玟̃obt@p
void  rendering_next_buffer(void);
void  rendering_next_buffer_asm(void);


/*-------------------------------*/
/* interrupt vector              */
/*-------------------------------*/

/* --- DMA ]I荞݃xN^ */
#pragma interrupt INTDMA3 audioDoneDMA

/*---------------------------*/
/* implementatiion               */
/*-------------------------------*/

/*------------------------------------------------------*/
/* interface                                            */
/*------------------------------------------------------*/

/* --- komeAudioInitialize() */
/**
 * I[fBIhCȍ
 * DAC̃|[gݒ
 * 
 * @return Ȃ AUDIO_DONEAȊOŎs
 *
 */
int komeAudioInitialize(void)
{
  /* --- DAC port initialize */
  PM1 = 0xff;

  return(AUDIO_DONE);
}


/* --- komeAudioStart() */
/**
 * I[fBIhCo̓X^[g
 * ^C}DMAAĊ荞݂Jn
 * 
 * @return Ȃ AUDIO_DONEAȊOŎs
 *
 */
int komeAudioStart(void)
{
	int i;
	unsigned long  ptr;

  /* --- first data */
  for(i=0; i<KOMEAUDIO_CHANNEL; i++) {
    se_channel[i].rest = 0;
    se_channel[i].repeat = 0;
  }
  audio_buffer_side = 1;
  rendering_next_buffer();
  audio_buffer_side = 0;
  rendering_next_buffer();

  /* --- DAC start */
  DA0M = 0x10;

  /* --- DMA start */
  /* source */
	ptr = (unsigned long)audio_buffer;
  DSA3H = ((ptr & 0x03ff0000) >> 16) | 0x8000;
  DSA3L = ptr & 0x0000ffff;
  /* distination (DA0CS0=0xfffff280) */
  DDA3H = 0x03ff;
  DDA3L = 0xf280;
  /* size */
  DBC3 = KOMEAUDIO_BUFFERSIZE - 1;
  /* control */
  DADC3 = 0b0000000000100000;
  /* trigger (INTTP2CC0) */
  DTFR3 = 0x15;
  /* start */
  DCHC3.0 = 1;

  /* --- timer start */
  TP2CTL0 = 0x00;
  TP2CTL1 = 0x00;
  /* 20MHz/8kh=625*4 */
  TP2CCR0 = 625*4;
  /* timer start */
  TP2CE = 1;
  
  /* --- enable interrupt */
  DMAIC3 = 0x03;
  __EI();

  return(AUDIO_DONE);
}


/* --- komeAudioStop() */
/**
 * I[fBIhCo̓~
 * DMA̒~DAC̒~
 * 
 * @return Ȃ AUDIO_DONEAȊOŎs
 *
 */
int komeAudioStop(void)
{
  int  i;

  /* --- DMA ~ */
  DCHC3.2 = 1;
  /* --- ~ */
  DA0CS0 = 0x80;
  DA0M = 0;
  /* --- channel stop */
  for(i=0; i<KOMEAUDIO_CHANNEL; i++) {
    se_channel[i].rest = 0;
    se_channel[i].repeat = 0;
  }
  return(AUDIO_DONE);
}


/* --- komeAudioSERequest() */
/**
 * `l̔NGXg
 * 
 * @param  ch [in] `l
 * @param  ptr [in] f[^|C^
 * @param  size [in] f[^TCY
 * @param  repeat [in] s[gtO (0=oneshot, 1=repeat)
 *
 * @return Ȃ AUDIO_DONEAȊOŎs
 *
 */
int komeAudioSERequest(int ch, unsigned char *ptr, int size, int repeat)
{
	int  align;

  /* --- assartion */
  if ((ch < 0) || (ch > KOMEAUDIO_CHANNEL)) {
    return(AUDIO_WRONG_CHANNEL);
  }
  
  /* --- channel start */
  align = (size + 3) & 0x7ffffffc;
  se_channel[ch].size = align;
  se_channel[ch].rest = align;
  se_channel[ch].data = ptr;
  se_channel[ch].repeat = repeat;

  return(AUDIO_DONE);
}


/* --- komeAudioSEStop() */
/**
 * `l̔~
 * 
 * @param  ch [in] ~`l
 *
 * @return Ȃ AUDIO_DONEAȊOŎs
 *
 */
int komeAudioSEStop(int ch)
{
  /* --- assartion */
  if ((ch < 0) || (ch > KOMEAUDIO_CHANNEL)) {
    return(AUDIO_WRONG_CHANNEL);
  }
  /* --- channel stop */
  se_channel[ch].rest = 0;
  se_channel[ch].repeat = 0;

  return(AUDIO_DONE);
}


/* --- komeAudioChannelStatus() */
/**
 * `l̔󋵊mF
 * 
 * @param  ch [in] m肽`l
 *
 * @return 0Ȃ~AȊOœ쒆
 *
 */
int komeAudioChannelStatus(int ch)
{
	return(se_channel[ch].rest);
}


/*------------------------------------------------------*/
/* Interrupt                                            */
/*------------------------------------------------------*/

/* --- DMA]̊荞ݏ     */
/*     DMA]vƃobt@̗p */
__interrupt void audioDoneDMA()
{
  unsigned char  *buffer;
  unsigned long  ptr;

	/* swap buffer */
	if (audio_buffer_side == 0) {
		audio_buffer_side = 1;
    buffer = audio_buffer + KOMEAUDIO_BUFFERSIZE;
	}
	else {
		audio_buffer_side = 0;
    buffer = audio_buffer;
	}

  /* --- DMA]Jn */
  /* set register */
	ptr = (unsigned long)buffer;
  DSA3H = ((ptr & 0x03ff0000) >> 16) | 0x8000;
  DSA3L = ptr & 0x0000ffff;
  /* size */
  DBC3 = KOMEAUDIO_BUFFERSIZE - 1;
  /* start */
  DCHC3.0 = 1;

  /* --- ̃obt@f[^~LVO */
  rendering_next_buffer_asm();
}


/*------------------------------------------------------*/
/* Local routine                                        */
/*------------------------------------------------------*/

/* --- DMA]ĂԂɎ̃obt@p(asm) */
void  rendering_next_buffer_asm(void)
{
  int  i;
  unsigned long *ptr;

  work_loop = KOMEAUDIO_BUFFERSIZE / 4;
  work_buffer = (unsigned long *)(audio_buffer + (KOMEAUDIO_BUFFERSIZE * (audio_buffer_side ^ 1)));

  /* --- obt@NA */
  ptr = work_buffer;
  for(i=0; i<work_loop; i++) {
    *ptr = 0x80808080;
    ptr++;
  }

  /* --- `l~LVO */
  for(i=0; i<KOMEAUDIO_CHANNEL; i++) {
    work_data_rest = se_channel[i].rest;
    if (work_data_rest == 0) continue;
    work_data = (unsigned long *)se_channel[i].data;
    work_data_size = se_channel[i].size;
#pragma asm
		pushm   r10,r11,r12,r13,r14,r15,r16,r17,r18,r19
    ld.w    $_work_loop, r10        -- r10 = loop counter
    ld.w    $_work_buffer, r11
    ld.w    $_work_data, r12
    ld.w    $_work_data_rest, r13
    ld.w    $_work_data_size, r14
    sub      r13, r14
    add      r14, r12
mix_loop:
    ld.w    0[r11], r14              -- r14 = buffer data
    ld.w    0[r12], r15              -- r15 = wave data
    -- add with limit
    mov     0x80808080, r16         -- r16 = add mask
    and      r15, r16
    mov      r16, r17
    shr      7, r17
    subr    r16, r17
    or      r17, r16
    mov      0x80808080, r17
    and      r16, r17
    subr    r15, r17
    and      r16, r17                -- r17 = add value
    mov      r14, r18
    and      r17, r18
    mov      r14, r19
    xor      r17, r19
    shr      1, r19
    and      0x7f7f7f7f, r19
    add      r19, r18
    and      0x80808080, r18
    mov      r18, r19
    shr      7, r19
    subr    r18, r19
    or      r19, r18                -- ov mask
    add      r17, r14
    sub      r18, r14
    or      r18, r14
    -- sub with limit
    xor      0xffffffff, r16          -- r16 = sub mask
    mov      0x80808080, r17
    sub      r15, r17
    and      r16, r17                -- r17 = sub value
    not      r14, r18
    and      r17, r18
    not      r14, r19
    xor      r17, r19
    shr      1, r19
    and      0x7f7f7f7f, r19
    add      r19, r18
    and      0x80808080, r18
    mov      r18, r19
    shr      7, r19
    subr    r18, r19
    or      r19, r18                -- ov mask
    or      r18, r17
    or      r18, r14
    sub      r17, r14
    -- str
    st.w    r14, 0[r11]    
    -- next    
    add      4, r11
    add      4, r12
    sub      4, r13
    jz      mix_end
    sub      1, r10
    jnz      mix_loop
mix_end:
    st.w    r13, $_work_data_rest
		popm   r10,r11,r12,r13,r14,r15,r16,r17,r18,r19
#pragma endasm    
    se_channel[i].rest = work_data_rest;
  }  
}


/* --- DMA]ĂԂɎ̃obt@pӂĂ */
void  rendering_next_buffer(void)
{
  int  i, ch;
  int  buf_ptr;
  int  tmp_buffer[KOMEAUDIO_BUFFERSIZE];
  int  wav, flag;
  unsigned char  byte;

  /* --- obt@NA */
  if (audio_buffer_side == 0) {
    buf_ptr = KOMEAUDIO_BUFFERSIZE;
  }
  else {
    buf_ptr = 0;
  }
  for(i=0; i<KOMEAUDIO_BUFFERSIZE; i++) {
    tmp_buffer[i] = 0;
  }

  /* --- `l~LVO */
  flag = 0;
  for(ch=0; ch<KOMEAUDIO_CHANNEL; ch++) {
    /* - łȂΎ */
    if (se_channel[ch].rest <= 0) continue;
    flag = 1;
    /* - ~LVO */
    for(i=0; i<KOMEAUDIO_BUFFERSIZE; i++) {
      byte = *(se_channel[ch].data + (se_channel[ch].size - se_channel[ch].rest));
      wav = ((int)byte) - 128;
      tmp_buffer[i] += wav;
      se_channel[ch].rest -= 1;
      if (se_channel[ch].rest <= 0) {
        if (se_channel[ch].repeat == 0) {
          break;
        }
        else {
          se_channel[ch].rest = se_channel[ch].size;
        }
      }
    }
  }

  /* --- ʎq */
  if (flag == 0) {
    for(i=0; i<KOMEAUDIO_BUFFERSIZE; i++) {
       audio_buffer[buf_ptr+i] = 0x80;
    }
  }
  else {
    for(i=0; i<KOMEAUDIO_BUFFERSIZE; i++) {
      wav = tmp_buffer[i] + 128;
      if (wav > 255) {
        wav = 255;
      }
      if (wav < 0) {
        wav = 0;
      }
      audio_buffer[buf_ptr+i] = (unsigned char)wav;
    }
  }
}
