/* ---------------------------------------------------------- */
/*  lcd_module.c                                              */
/* ---------------------------------------------------------- */

/*------------------------------------------------------------- */
/** @file
    @brief		V850/LCD module controller
    @author		rerofumi
    @since		Apr.14.2007
*/
/*-----------------------------------------------------
 Copyright (C) 2007 rerofumi
 All Rights Reserved.
 ------------------------------------------------------*/

#pragma ioreg

/*-------------------------------*/
/* include                       */
/*-------------------------------*/
#include "lcd_module.h"

/*-------------------------------*/
/* define                        */
/*-------------------------------*/

/* --- 16i\p */
const unsigned char hex_char[16] = {
		'0','1','2','3','4','5','6','7',
		'8','9','A','B','C','D','E','F' };

/*-------------------------------*/
/* global value                  */
/*-------------------------------*/

char lcd_enable;

/*-------------------------------*/
/* local routine                 */
/*-------------------------------*/

/*-------------------------------*/
/* implementatiion               */
/*-------------------------------*/

/*------------------------------------------------------*/
/* interface                                            */
/*------------------------------------------------------*/

/* --- lcdInitialize() */
/**
 * LCD W[̏
 * 
 * @return Ȃ LCD_DONEAȊOŎs
 *
 */
int	lcdInitialize(void)
{
	/* --- waitpɃC^[o^C}[ݒ */
	/* - div=clk/4, compare=5000, 1ms */
	TM0CTL0 = 0x02;
	TM0CMP0 = 5000;
	/* - C^[o^C}[͒~Ă */
	TM0CE = 0;

	/* --- |[gݒ */
	LCD_PORT_MODE = (unsigned char)~LCD_PORT_MASK;	
	LCD_PORT2_MODE = (unsigned char)~LCD_PORT2_MASK;	

	wait_1ms(20);
	lcd_enable = 0;

	/* --- step 1 */
	LCD_RW = 0;
	LCD_RS = 0;
	LCD_DB4 = LCD_DB5 = 1;
	LCD_DB6 = LCD_DB7 = 0;
	LCD_E = 1;
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	LCD_E = 0;

	wait_1ms(11);
	
	/* -- step 2 */	
	LCD_RW = 0;
	LCD_RS = 0;
	LCD_DB4 = LCD_DB5 = 1;
	LCD_DB6 = LCD_DB7 = 0;
	LCD_E = 1;
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	LCD_E = 0;

	wait_1ms(1);

	/* -- step 3 */	
	LCD_RW = 0;
	LCD_RS = 0;
	LCD_DB4 = LCD_DB5 = 1;
	LCD_DB6 = LCD_DB7 = 0;
	LCD_E = 1;
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	LCD_E = 0;
	
	/* --- busy wait */
	lcd_enable = lcdWaitBusy();

	/* --- function set */
	lcdSendCommand(0x28);
	/* --- display off */
	lcdSendCommand(0x08);
	/* --- display clear */
	lcdSendCommand(0x01);
	wait_1ms(2);
	/* --- entry mode */
	lcdSendCommand(0x06);
	/* --- display start */
#if (LCD_MODE_DISPCURSOR == 1)
	lcdSendCommand(0x0e);
#else
	lcdSendCommand(0x0c);
#endif

	return(lcd_enable);
}


/* --- lcdWaitBusy() */
/**
 * LCD W[ Busy ҂
 * 
 * @return Ȃ LCD_DONEAȊOŎs
 *
 */
int lcdWaitBusy(void)
{
	int		bf;

	/* --- |[gǂݍݐݒ */	
	LCD_PORT_MODE |= LCD_PORT_MASK;	

	/* - C^[o^C}[X^[g */
	TM0CMP0 = 15000;
	TM0EQIF0 = 0;
	TM0CE = 1;
	
	do {
		LCD_RW = 1;
		LCD_RS = 0;
		LCD_E = 1;
		__asm("nop");
		__asm("nop");
		__asm("nop");
		__asm("nop");
		__asm("nop");
		__asm("nop");
		bf = LCD_DB7;
		LCD_E = 0;
		__asm("nop");
		__asm("nop");
		LCD_E = 1;
		__asm("nop");
		__asm("nop");
		__asm("nop");
		__asm("nop");
		__asm("nop");
		__asm("nop");
		LCD_E = 0;
		if (bf == 0) {
			break;
		}
	} while(TM0EQIF0 == 0);
	TM0CE = 0;
	TM0CMP0 = 5000;
	LCD_PORT_MODE &= ~LCD_PORT_MASK;	

	if (bf != 0) {
		return(LCD_ERROR);
	}
	else {
		return(LCD_DONE);
	}
}


/* --- lcdSendCommand() */
/**
 * LCD W[ɃR}hM
 * 
 * @param com [in] MR}h
 * @return Ȃ LCD_DONEAȊOŎs
 *
 */
int lcdSendCommand(unsigned char com)
{
	if (lcdWaitBusy() != LCD_DONE) {
		return(LCD_ERROR);
	}
		
	LCD_RW = 0;
	LCD_RS = 0;
	LCD_DB7 = (com >> 7) & 0x01;
	LCD_DB6 = (com >> 6) & 0x01;
	LCD_DB5 = (com >> 5) & 0x01;
	LCD_DB4 = (com >> 4) & 0x01;
	LCD_E = 1;
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	LCD_E = 0;
	__asm("nop");
	__asm("nop");
	LCD_DB7 = (com >> 3) & 0x01;
	LCD_DB6 = (com >> 2) & 0x01;
	LCD_DB5 = (com >> 1) & 0x01;
	LCD_DB4 = com & 0x01;
	LCD_E = 1;
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	LCD_E = 0;

	return(LCD_DONE);
}


/* --- lcdReceiveByte() */
/**
 * LCD W[1byteǂݍ
 * 
 * @param dat [out] Mf[^߂|C^
 * @return Ȃ LCD_DONEAȊOŎs
 *
 */
int lcdReceiveByte(unsigned char *dat)
{
	if (lcdWaitBusy() != LCD_DONE) {
		return(LCD_ERROR);
	}

	/* --- |[gǂݍݐݒ */	
	LCD_PORT_MODE |= LCD_PORT_MASK;	

	*dat = 0;		
	LCD_RW = 1;
	LCD_RS = 1;
	LCD_E = 1;
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	*dat |= LCD_DB7;
	*dat <<= 1;
	*dat |= LCD_DB6;
	*dat <<= 1;
	*dat |= LCD_DB5;
	*dat <<= 1;
	*dat |= LCD_DB4;
	*dat <<= 1;
	LCD_E = 0;
	__asm("nop");
	__asm("nop");
	LCD_E = 1;
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	*dat |= LCD_DB7;
	*dat <<= 1;
	*dat |= LCD_DB6;
	*dat <<= 1;
	*dat |= LCD_DB5;
	*dat <<= 1;
	*dat |= LCD_DB4;
	LCD_E = 0;

	LCD_PORT_MODE &= ~LCD_PORT_MASK;	

	return(LCD_DONE);
}


/* --- lcdClearScreen() */
/**
 * ʃNA
 * 
 * @return Ȃ LCD_DONEAȊOŎs
 *
 */
int lcdClearScreen(void)
{
	return(lcdSendCommand(0x01));
}


/* --- lcdLocate() */
/**
 * \ʒuw
 * 
 * @param x [in] J[\ړʒu x
 * @param y [in] J[\ړʒu y
 * @return Ȃ LCD_DONEAȊOŎs
 *
 */
int lcdLocate(int x, int y)
{
	unsigned char  com;
	
	com = x + ((y % 2) * 0x40) + ((y / 2) * 0x20);
	com |= 0x80;
	return(lcdSendCommand(com));
}


/* --- lcdPutChar() */
/**
 * 1M
 * 
 * @param dat [in] \LN^[R[h
 * @return Ȃ LCD_DONEAȊOŎs
 *
 */
int lcdPutChar(unsigned char dat)
{
	if (lcdWaitBusy() != LCD_DONE) {
		return(LCD_ERROR);
	}
		
	LCD_RW = 0;
	LCD_RS = 1;
	LCD_DB7 = (dat >> 7) & 0x01;
	LCD_DB6 = (dat >> 6) & 0x01;
	LCD_DB5 = (dat >> 5) & 0x01;
	LCD_DB4 = (dat >> 4) & 0x01;
	LCD_E = 1;
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	LCD_E = 0;
	__asm("nop");
	LCD_DB7 = (dat >> 3) & 0x01;
	LCD_DB6 = (dat >> 2) & 0x01;
	LCD_DB5 = (dat >> 1) & 0x01;
	LCD_DB4 = dat & 0x01;
	LCD_E = 1;
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	__asm("nop");
	LCD_E = 0;

	return(LCD_DONE);
}


/* --- lcdPutHexByte() */
/**
 * 1byte hex \
 * 
 * @param num [in] \l
 * @return Ȃ LCD_DONEAȊOŎs
 *
 */
int lcdPutHexByte(unsigned char num)
{
	unsigned char  c;
	
	c = (num >> 4) & 0x0f;
	lcdPutChar(hex_char[c]);
	c = num & 0x0f;
	lcdPutChar(hex_char[c]);

	return(LCD_DONE);
}


/* --- lcdPutHexShort() */
/**
 * 2byte hex \
 * 
 * @param num [in] \l
 * @return Ȃ LCD_DONEAȊOŎs
 *
 */
int lcdPutHexShort(unsigned short num)
{
	unsigned char  c;
	
	c = (num >> 12) & 0x0f;
	lcdPutChar(hex_char[c]);
	c = (num >> 8) & 0x0f;
	lcdPutChar(hex_char[c]);
	c = (num >> 4) & 0x0f;
	lcdPutChar(hex_char[c]);
	c = num & 0x0f;
	lcdPutChar(hex_char[c]);

	return(LCD_DONE);
}


/* --- lcdPutMessage() */
/**
 * \
 * 
 * @param string [in] \镶ւ̃|C^
 * @return Ȃ LCD_DONEAȊOŎs
 * @note 񒆂 \0 oĂ܂ŕ\
 *
 */
int lcdPutMessage(unsigned char *string)
{
	int  count;
	
	count = 80;
	while(*string != 0) {
		lcdPutChar(*string);
		string += 1;
		count -= 1;
		if (count == 0) break;
	}

	return(LCD_DONE);
}



/*------------------------------------------------------*/
/* local routine                                        */
/*------------------------------------------------------*/

/* --- wait_1ms() */
/**
 * 1ms ̃EFCg[`
 * Ɏg
 * @param msec [in] EFCgԁAPʃ~b
 */
void  wait_1ms(int msec)
{
	int  i;

	/* - C^[o^C}[X^[g */
	TM0CE = 1;

	for(i=0; i<msec; i++) {	
  	TM0EQIF0 = 0;
		while(TM0EQIF0 == 0) {
			__asm("nop");
		}
	}
	
	TM0CE = 0;
}


