#pragma ioreg


#include "kome_audio_drv.h"
#include "kome_input_drv.h"
#include "lcd_module.h"


/* --------------------------------------------- */
/* --- static data                               */
/* --------------------------------------------- */

#include "voice.h"


/* --------------------------------------------- */
/* --- define                                    */
/* --------------------------------------------- */

#undef  LCD_ENABLE

/* --------------------------------------------- */
/* --- main ui                                   */
/* --------------------------------------------- */

void vout_pingpong(void);
void vout_hour(int h);
void vout_min(int m);

/*-------------------------------*/
/* interrupt vector              */
/*-------------------------------*/

/* --- 1sec ^C}[荞݃xN^ */
#pragma interrupt INTTP1CC0 clockCounter

/*-------------------------------*/
/* static value                  */
/*-------------------------------*/

static  int  int_counter;
static  int  time_h, time_m, time_s;

/* --------------------------------------------- */
/* --- main ui                                   */
/* --------------------------------------------- */

void main( void )
{
	int  post_sec, h, m;
	int  inkey, postkey;

	/* CPU */
	VSWC	= 1;
	WDTM2	= 0;

	__asm("mov	0x00, r11");
	__asm("st.b	r11,  PRCMD");
	__asm("st.b	r11,  PCC");

	while( LOCK );
	SELPLL	= 1;

	komeInputInitialize();
	komeAudioInitialize();
	komeAudioStart();

	/* --- time clear */
	time_h = time_m = time_s = 0;

	/* --- 1sec counter */
	int_counter = 0;
  __DI();
  TP1CTL0 = 0x06;
  TP1CTL1 = 0x00;
  TP1CCR0 = 15625;  // 0.2sec
  TP1IOC0 = 0;
  /* timer start */
  TP1CE = 1;
	
  /* --- enable interrupt */
  TP1CCIC0 = 0x04;
  __EI();
		
	/* --------------------------------------------- */
	/* --- main ui                                   */
	/* --------------------------------------------- */

	/* --- AvP[V */
	post_sec = time_s;
	postkey = 0;

	/* --- LCD  */
#ifdef LCD_ENABLE
	lcdInitialize();
	lcdClearScreen();
#endif	

	while(1) {
		komeInputRead(&inkey);
		if (((inkey & KIN_Left) != 0) && ((postkey & KIN_Left) == 0)) {
			time_h += 1;
			if (time_h >= 24) {
				time_h = 0;
			}
			vout_hour(time_h);
		}
		if (((inkey & KIN_Right) != 0) && ((postkey & KIN_Right) == 0)) {
			time_m += 1;
			if (time_m >= 60) {
				time_m = 0;
			}
			vout_min(time_m);
		}
		if (((inkey & KIN_Button1) != 0) && ((postkey & KIN_Button1) == 0)) {
			h = time_h;
			m = time_m;
			vout_pingpong();
			vout_hour(h);
			vout_min(m);
		}
		postkey = inkey;

		/* -- next */
#ifdef LCD_ENABLE
		if (post_sec != time_s) {
			lcdLocate(0, 0);
			lcdPutHexByte(time_h);
			lcdLocate(2, 0);
			lcdPutChar(':');
			lcdLocate(3, 0);
			lcdPutHexByte(time_m);
			lcdLocate(5, 0);
			lcdPutChar(':');
			lcdLocate(6, 0);
			lcdPutHexByte(time_s);
			post_sec = time_s;
		}
#endif
		wait_1ms(100);
	}

}


/* --------------------------------------------- */
/* --- timer intterupt (0.2sec)                  */
/* --------------------------------------------- */
__interrupt void clockCounter()
{
	int_counter += 1;
	if (int_counter > 4) {
		int_counter = 0;
		/* --- 1sec */
		time_s += 1;
		if (time_s >= 60) {
			time_s = 0;
			time_m += 1;
			if (time_m >= 60) {
				time_m = 0;
				time_h += 1;
				if (time_h >= 24) {
					time_h = 0;
				}
			}
		}
	}
}


/* --------------------------------------------- */
/* --- voice output                              */
/* --------------------------------------------- */

/* ---------------------------------- */
/* -- in */
void vout_pingpong(void)
{
	 komeAudioSERequest(0, (unsigned char *)vcin, L_VCIN, 0);
	 while( komeAudioChannelStatus(0) != 0) {
		 __asm("nop");
	 }
}


/* ---------------------------------- */
/* -- hour */
void vout_hour(int h)
{
	int  i, j;

	i = h / 10;
	switch(i) {
		case 1:
	 		komeAudioSERequest(0, (unsigned char *)vc10, L_VC10, 0);
	 		break;
		case 2:
	 		komeAudioSERequest(0, (unsigned char *)vc20, L_VC20, 0);
	 		break;
	 	default:		
	 		break;
	}
	while( komeAudioChannelStatus(0) != 0) {
		__asm("nop");
	}
	j = h % 10;
	if (((i > 0) && (j == 0)) == 0) {
		switch(j) {
		case 0:
	 		komeAudioSERequest(0, (unsigned char *)vc00, L_VC00, 0);
	 		break;
		case 1:
	 		komeAudioSERequest(0, (unsigned char *)vc01, L_VC01, 0);
	 		break;
		case 2:
	 		komeAudioSERequest(0, (unsigned char *)vc02, L_VC02, 0);
	 		break;
		case 3:
	 		komeAudioSERequest(0, (unsigned char *)vc03, L_VC03, 0);
	 		break;
		case 4:
	 		komeAudioSERequest(0, (unsigned char *)vc04, L_VC04, 0);
	 		break;
		case 5:
	 		komeAudioSERequest(0, (unsigned char *)vc05, L_VC05, 0);
	 		break;
		case 6:
	 		komeAudioSERequest(0, (unsigned char *)vc06, L_VC06, 0);
	 		break;
		case 7:
	 		komeAudioSERequest(0, (unsigned char *)vc07, L_VC07, 0);
	 		break;
		case 8:
	 		komeAudioSERequest(0, (unsigned char *)vc08, L_VC08, 0);
	 		break;
		case 9:
	 		komeAudioSERequest(0, (unsigned char *)vc09, L_VC09, 0);
	 		break;
	 	default:		
	 		break;
		}
	}
	while( komeAudioChannelStatus(0) != 0) {
		__asm("nop");
	}
	komeAudioSERequest(0, (unsigned char *)vchour, L_VCHOUR, 0);
	while( komeAudioChannelStatus(0) != 0) {
		__asm("nop");
	}
}


/* ---------------------------------- */
/* -- min */
void vout_min(int m)
{
	int  i, j;

	i = m / 10;
	switch(i) {
		case 1:
	 		komeAudioSERequest(0, (unsigned char *)vc10, L_VC10, 0);
	 		break;
		case 2:
	 		komeAudioSERequest(0, (unsigned char *)vc20, L_VC20, 0);
	 		break;
		case 3:
	 		komeAudioSERequest(0, (unsigned char *)vc30, L_VC30, 0);
	 		break;
		case 4:
	 		komeAudioSERequest(0, (unsigned char *)vc40, L_VC40, 0);
	 		break;
		case 5:
	 		komeAudioSERequest(0, (unsigned char *)vc50, L_VC50, 0);
	 		break;
	 	default:		
	 		break;
	}
	while( komeAudioChannelStatus(0) != 0) {
		__asm("nop");
	}
	j = m % 10;
	if (((i > 0) && (j == 0)) == 0) {
		switch(j) {
		case 0:
	 		komeAudioSERequest(0, (unsigned char *)vc00, L_VC00, 0);
	 		break;
		case 1:
	 		komeAudioSERequest(0, (unsigned char *)vc01, L_VC01, 0);
	 		break;
		case 2:
	 		komeAudioSERequest(0, (unsigned char *)vc02, L_VC02, 0);
	 		break;
		case 3:
	 		komeAudioSERequest(0, (unsigned char *)vc03, L_VC03, 0);
	 		break;
		case 4:
	 		komeAudioSERequest(0, (unsigned char *)vc04, L_VC04, 0);
	 		break;
		case 5:
	 		komeAudioSERequest(0, (unsigned char *)vc05, L_VC05, 0);
	 		break;
		case 6:
	 		komeAudioSERequest(0, (unsigned char *)vc06, L_VC06, 0);
	 		break;
		case 7:
	 		komeAudioSERequest(0, (unsigned char *)vc07, L_VC07, 0);
	 		break;
		case 8:
	 		komeAudioSERequest(0, (unsigned char *)vc08, L_VC08, 0);
	 		break;
		case 9:
	 		komeAudioSERequest(0, (unsigned char *)vc09, L_VC09, 0);
	 		break;
	 	default:		
	 		break;
		}
	}
	while( komeAudioChannelStatus(0) != 0) {
		__asm("nop");
	}
	komeAudioSERequest(0, (unsigned char *)vcmin, L_VCMIN, 0);
	while( komeAudioChannelStatus(0) != 0) {
		__asm("nop");
	}
}

